==> ircii-2.8.43-pana.tar.gz: ircii-2.8/source/flood.c <==
/*
 * flood.c: handle channel flooding. 
 *
 * This attempts to give you some protection from flooding.  Basically, it keeps
 * track of how far apart (timewise) messages come in from different people.
 * If a single nickname sends more than 3 messages in a row in under a
 * second, this is considered flooding.  It then activates the ON FLOOD with
 * the nickname and type (appropriate for use with IGNORE). 
 *
 * Thanks to Tomi Ollila <f36664r@puukko.hut.fi> for this one. 
 */


#include "irc.h"

#include "alias.h"
#include "hook.h"
#include "ircaux.h"
#include "ignore.h"
#include "flood.h"
#include "vars.h"
#include "output.h"
#include "list.h"
#include "pluto.h"
#include "server.h"
#include "userlist.h"

static	char	*ignore_types[NUMBER_OF_FLOODS] =
{
	"MSG",
	"PUBLIC",
	"NOTICE",
	"WALL",
	"WALLOP",
	"CTCP",
	"INVITE",
};

typedef struct flood_stru
{
	char	nick[NICKNAME_LEN + 1];
	int	type;
	char	flood;
	long	cnt;
	time_t	start;
}	Flooding;

static Flooding *flood = NULL;

extern	char	*FromUserHost;
extern	unsigned int window_display;
extern	int	from_server;

#define NO_RESET 0
#define RESET 1

int get_flood_rate(int type, ChannelList * channel)
{
	int flood_rate = get_int_var(FLOOD_RATE_VAR);
	if (channel)
	{
		switch(type)
		{
			case PUBLIC_FLOOD:
				flood_rate = channel->set_pubflood_time;
				break;
			case NICK_FLOOD:
				flood_rate = channel->set_nickflood_time;
				break;
			case KICK_FLOOD:
				flood_rate = channel->set_kickflood_time;
				break;
			case DEOP_FLOOD:
				flood_rate = channel->set_deopflood_time;
				break;
			default:
				break;
		}
	}
	return flood_rate;
}

int get_flood_count(int type, ChannelList * channel)
{
	int flood_count = get_int_var(FLOOD_AFTER_VAR);
	if (channel) {
		switch(type)
		{
			case PUBLIC_FLOOD:
				flood_count = channel->set_kick_on_pubflood;
				break;
			case NICK_FLOOD:
				flood_count = channel->set_kick_on_nickflood;
				break;
			case KICK_FLOOD:
				flood_count = channel->set_kick_on_kickflood;
				break;
			case DEOP_FLOOD:
				flood_count = channel->set_deop_on_deopflood;
				break;
			default:
			break;
		}
	}
	return flood_count;
}

int set_nick_flood(int type, time_t flood_time, int reset, NickList *tmpnick)
{
	if (!tmpnick)
		return 0;
	switch(type)
	{
		case PUBLIC_FLOOD:
			(reset == RESET)? tmpnick->floodcount = 1: tmpnick->floodcount++;
			tmpnick->floodtime = flood_time;
			break;
		case NICK_FLOOD:
			(reset == RESET)? tmpnick->nickcount = 1: tmpnick->nickcount++;
			tmpnick->nicktime = flood_time;
			break;
		case KICK_FLOOD:
			(reset == RESET)? tmpnick->kickcount = 1: tmpnick->kickcount++;
			tmpnick->kicktime = flood_time;
			break;
		case DEOP_FLOOD:
			(reset == RESET)? tmpnick->dopcount = 1: tmpnick->dopcount++;
			tmpnick->doptime = flood_time;
			break;
		default:
		break;
	}
	return 1;
}

int is_other_flood(ChannelList *channel, NickList *tmpnick, int type)
{
time_t diff = 0, flood_time;
int doit = 0;
int count = 0;
int flood_rate, flood_count;

	flood_time = time(NULL);

	if (channel && tmpnick)
	{
		
		switch(type)
		{
			case PUBLIC_FLOOD:
				if (!channel->set_pubflood)
					break;
				diff = flood_time - tmpnick->floodtime;
				doit = 1; count = tmpnick->floodcount;
				break;
			case NICK_FLOOD:
				if (!channel->set_nickflood)
					break;
				diff = flood_time - tmpnick->nicktime;
				doit = 1; count = tmpnick->nickcount;			
				break;
			case DEOP_FLOOD:
				if (!channel->set_deopflood)
					break;
				diff = flood_time - tmpnick->doptime;
				doit = 1; count = tmpnick->dopcount;
				break;
			case KICK_FLOOD:
				if (!channel->set_kickflood)
					break;
				diff = flood_time - tmpnick->kicktime;
				doit = 1; count = tmpnick->kickcount;
			default:
				break;
		}
		if (doit)
		{
			flood_count = get_flood_count(type, channel);
			flood_rate = get_flood_rate(type, channel);
			if (count > flood_count)
			{
				if ((diff == 0) || (count / diff) >= (flood_count /flood_rate))
				{
					if (tmpnick->userlevel < 90)
						return 1;
				}
				else
					set_nick_flood(type, flood_time, RESET, tmpnick);	
			}
		}
	}
	return 0;
} 

/*
 * check_flooding: This checks for message flooding of the type specified for
 * the given nickname.  This is described above.  This will return 0 if no
 * flooding took place, or flooding is not being monitored from a certain
 * person.  It will return 1 if flooding is being check for someone and an ON
 * FLOOD is activated. 
 */
int
check_flooding(nick, type, line)
	char	*nick;
	int	type;
	char	*line;
{
	static	int	users = 0,
			pos = 0;
	int	i;
	time_t	flood_time, diff;
	Flooding *tmp;
	int flooder = 0;
	int flood_rate, flood_count;				
	if (users != get_int_var(FLOOD_USERS_VAR))
	{
		if ((users = get_int_var(FLOOD_USERS_VAR)) == 0)
			return(1);
		if (flood)
			flood = (Flooding *) new_realloc((char *) flood, sizeof(Flooding) * users);
		else
			flood = (Flooding *) new_malloc(sizeof(Flooding) *
				users);
		for (i = 0; i < users; i++)
		{
			*(flood[i]).nick = 0;
			flood[i].cnt = 0;
			flood[i].flood = 0;
		}
	}
	if (users == 0)
		return (1);
	for (i = 0; i < users; i++)
	{
		if (*(flood[i].nick))
		{
			if ((my_stricmp(nick, flood[i].nick) == 0) &&
					(type == flood[i].type))
				break;
		}
	}
	flood_time = time(NULL);
	if (i == users)
	{
		tmp = &(flood[pos]);
		pos = (pos + 1) % users;
		strmcpy(tmp->nick, nick, NICKNAME_LEN);
		tmp->type = type;
		tmp->cnt = 1;
		tmp->start = flood_time;
		tmp->flood = 0;
		return (1);
	}
	else
		tmp = &(flood[i]);

	tmp->cnt++;
	diff = flood_time - tmp->start;
	flood_count = get_flood_count(type, NULL);
	flood_rate = get_flood_rate(type, NULL);

	if (tmp->cnt > flood_count)
	{
		if ((diff == 0) || (tmp->cnt / diff) > flood_rate)
		{
			if (tmp->flood == 0)
			{
				if (get_int_var(FLOOD_WARNING_VAR))
					say("%s flooding detected from %s", ignore_types[type], nick);
				tmp->flood = 1;
			}
			flooder = do_hook(FLOOD_LIST, "%s %s %s", nick, ignore_types[type],line);
			return(flooder);
		}
		else
		{
			tmp->flood = 0;
			tmp->cnt = 1;
			tmp->start = flood_time;
		}
	}
	return (1);
}

==> ircii-2.8.45-pana.tar.gz: ircii-2.8/source/flood.c <==
/*
 * flood.c: handle channel flooding. 
 *
 * This attempts to give you some protection from flooding.  Basically, it keeps
 * track of how far apart (timewise) messages come in from different people.
 * If a single nickname sends more than 3 messages in a row in under a
 * second, this is considered flooding.  It then activates the ON FLOOD with
 * the nickname and type (appropriate for use with IGNORE). 
 *
 * Thanks to Tomi Ollila <f36664r@puukko.hut.fi> for this one. 
 */


#include "irc.h"

#include "alias.h"
#include "hook.h"
#include "ircaux.h"
#include "ignore.h"
#include "flood.h"
#include "vars.h"
#include "output.h"
#include "list.h"
#include "pluto.h"
#include "server.h"
#include "userlist.h"

static	char	*ignore_types[NUMBER_OF_FLOODS] =
{
	"MSG",
	"PUBLIC",
	"NOTICE",
	"WALL",
	"WALLOP",
	"CTCP",
	"INVITE",
};

typedef struct flood_stru
{
	char	nick[NICKNAME_LEN + 1];
	int	type;
	char	flood;
	long	cnt;
	time_t	start;
}	Flooding;

static Flooding *flood = NULL;

extern	char	*FromUserHost;
extern	unsigned int window_display;
extern	int	from_server;

#define NO_RESET 0
#define RESET 1

int get_flood_rate(int type, ChannelList * channel)
{
	int flood_rate = get_int_var(FLOOD_RATE_VAR);
	if (channel)
	{
		switch(type)
		{
			case PUBLIC_FLOOD:
				flood_rate = channel->set_pubflood_time;
				break;
			case NICK_FLOOD:
				flood_rate = channel->set_nickflood_time;
				break;
			case KICK_FLOOD:
				flood_rate = channel->set_kickflood_time;
				break;
			case DEOP_FLOOD:
				flood_rate = channel->set_deopflood_time;
				break;
			default:
				break;
		}
	}
	return flood_rate;
}

int get_flood_count(int type, ChannelList * channel)
{
	int flood_count = get_int_var(FLOOD_AFTER_VAR);
	if (channel) {
		switch(type)
		{
			case PUBLIC_FLOOD:
				flood_count = channel->set_kick_on_pubflood;
				break;
			case NICK_FLOOD:
				flood_count = channel->set_kick_on_nickflood;
				break;
			case KICK_FLOOD:
				flood_count = channel->set_kick_on_kickflood;
				break;
			case DEOP_FLOOD:
				flood_count = channel->set_deop_on_deopflood;
				break;
			default:
			break;
		}
	}
	return flood_count;
}

int set_nick_flood(int type, time_t flood_time, int reset, NickList *tmpnick)
{
	if (!tmpnick)
		return 0;
	switch(type)
	{
		case PUBLIC_FLOOD:
			(reset == RESET)? tmpnick->floodcount = 1: tmpnick->floodcount++;
			tmpnick->floodtime = tmpnick->idle_time = flood_time;
			break;
		case NICK_FLOOD:
			(reset == RESET)? tmpnick->nickcount = 1: tmpnick->nickcount++;
			tmpnick->nicktime = flood_time;
			break;
		case KICK_FLOOD:
			(reset == RESET)? tmpnick->kickcount = 1: tmpnick->kickcount++;
			tmpnick->kicktime = flood_time;
			break;
		case DEOP_FLOOD:
			(reset == RESET)? tmpnick->dopcount = 1: tmpnick->dopcount++;
			tmpnick->doptime = flood_time;
			break;
		default:
		break;
	}
	return 1;
}

int is_other_flood(ChannelList *channel, NickList *tmpnick, int type)
{
time_t diff = 0, flood_time;
int doit = 0;
int count = 0;
int flood_rate, flood_count;

	flood_time = time(NULL);

	if (channel && tmpnick)
	{
		
		switch(type)
		{
			case PUBLIC_FLOOD:
				if (!channel->set_pubflood)
					break;
				diff = flood_time - tmpnick->floodtime;
				doit = 1; count = tmpnick->floodcount;
				break;
			case NICK_FLOOD:
				if (!channel->set_nickflood)
					break;
				diff = flood_time - tmpnick->nicktime;
				doit = 1; count = tmpnick->nickcount;			
				break;
			case DEOP_FLOOD:
				if (!channel->set_deopflood)
					break;
				diff = flood_time - tmpnick->doptime;
				doit = 1; count = tmpnick->dopcount;
				break;
			case KICK_FLOOD:
				if (!channel->set_kickflood)
					break;
				diff = flood_time - tmpnick->kicktime;
				doit = 1; count = tmpnick->kickcount;
			default:
				break;
		}
		if (doit)
		{
			flood_count = get_flood_count(type, channel);
			flood_rate = get_flood_rate(type, channel);
			if (count > flood_count)
			{
				if ((diff == 0) || (count / diff) >= (flood_count /flood_rate))
				{
					if (tmpnick->userlist && tmpnick->userlist->level < 90)
						return 1;
				}
				else
					set_nick_flood(type, flood_time, RESET, tmpnick);	
			}
		}
	}
	return 0;
} 

/*
 * check_flooding: This checks for message flooding of the type specified for
 * the given nickname.  This is described above.  This will return 0 if no
 * flooding took place, or flooding is not being monitored from a certain
 * person.  It will return 1 if flooding is being check for someone and an ON
 * FLOOD is activated. 
 */
int
check_flooding(nick, type, line)
	char	*nick;
	int	type;
	char	*line;
{
	static	int	users = 0,
			pos = 0;
	int	i;
	time_t	flood_time, diff;
	Flooding *tmp;
	int flooder = 0;
	int flood_rate, flood_count;				
	if (users != get_int_var(FLOOD_USERS_VAR))
	{
		if ((users = get_int_var(FLOOD_USERS_VAR)) == 0)
			return(1);
		if (flood)
			flood = (Flooding *) new_realloc((char *) flood, sizeof(Flooding) * users);
		else
			flood = (Flooding *) new_malloc(sizeof(Flooding) *
				users);
		for (i = 0; i < users; i++)
		{
			*(flood[i]).nick = 0;
			flood[i].cnt = 0;
			flood[i].flood = 0;
		}
	}
	if (users == 0)
		return (1);
	for (i = 0; i < users; i++)
	{
		if (*(flood[i].nick))
		{
			if ((my_stricmp(nick, flood[i].nick) == 0) &&
					(type == flood[i].type))
				break;
		}
	}
	flood_time = time(NULL);
	if (i == users)
	{
		tmp = &(flood[pos]);
		pos = (pos + 1) % users;
		strmcpy(tmp->nick, nick, NICKNAME_LEN);
		tmp->type = type;
		tmp->cnt = 1;
		tmp->start = flood_time;
		tmp->flood = 0;
		return (1);
	}
	else
		tmp = &(flood[i]);

	tmp->cnt++;
	diff = flood_time - tmp->start;
	flood_count = get_flood_count(type, NULL);
	flood_rate = get_flood_rate(type, NULL);

	if (tmp->cnt > flood_count)
	{
		if ((diff == 0) || (tmp->cnt / diff) > flood_rate)
		{
			if (tmp->flood == 0)
			{
				if (get_int_var(FLOOD_WARNING_VAR))
					say("%s flooding detected from %s", ignore_types[type], nick);
				tmp->flood = 1;
			}
			flooder = do_hook(FLOOD_LIST, "%s %s %s", nick, ignore_types[type],line);
			return(flooder);
		}
		else
		{
			tmp->flood = 0;
			tmp->cnt = 1;
			tmp->start = flood_time;
		}
	}
	return (1);
}
